from os.path import exists
from pathlib import Path

import click

from mpl_pymanage.utils import shell, find_python


def run_tests(test_folder, runner):

    if not exists(test_folder):
        raise click.BadParameter('No such folder %s' % test_folder)

    if runner == 'unittest':
        shell(find_python() + ' -m unittest discover ' + str(test_folder), silent=False)
    elif runner == 'pytest':
        shell(find_python() + ' -m pip install --upgrade pytest')
        shell(find_python() + ' -m pytest ' + str(test_folder), silent=False)
    else:
        raise click.BadParameter('No such test runner: %s' % runner)


def run_coverage(source, test_dir, runner):

    if not exists(test_dir):
        raise click.BadParameter('No such folder or file: %s' % test_dir)

    if not exists(source):
        raise click.BadParameter('No such folder %s' % source)

    if runner == 'unittest':
        shell(find_python() + ' -m pip install --upgrade coverage')
        shell(find_python() + ' -m coverage run --source=%s -m unittest discover %s' %
              (source, test_dir), silent=False)
    elif runner == 'pytest':
        shell(find_python() + ' -m pip install --upgrade coverage pytest')
        shell(find_python() + ' -m coverage run --source=%s -m pytest %s' %
              (source, test_dir), silent=False)
    else:
        raise click.BadParameter('No such test runner: %s' % runner)

    shell(find_python() + ' -m coverage report -m', silent=False)


def run_docs():
    """Build the documentation and show it in the browser."""

    cwd = Path.cwd()
    if not exists(cwd / 'docs'):
        raise click.BadParameter('Cannot find docs folder.')

    shell('make clean', root=str(cwd / 'docs'), silent=False)
    shell(find_python() + ' -m sphinx.cmd.build -M html . _build', root=str(cwd / 'docs'), silent=False)
    click.launch(str(cwd / 'docs' / '_build' / 'html' / 'index.html'))


def run_flake8(dirs):
    """Run flake8 style enforcement in specified directories.

    Parameters
    ----------
    dirs: list of str
        The directories to scan for style violations.
    """

    shell(find_python() + ' -m pip install --upgrade flake8')
    shell(find_python() + ' -m flake8 ' + ' '.join(dirs) + ' --max-line-length=120', silent=False)



from os.path import exists
from pathlib import Path
import shutil

import click

from mpl_pymanage.utils import shell, find_python


def deploy_pypi():
    """Deploy project in current directory to PyPi."""

    cwd = Path.cwd()

    if not exists(cwd / 'setup.py'):
        raise click.UsageError(
            'No setup.py found. Is this really a project root?')

    if exists(cwd / 'build') or exists(cwd / 'dist'):
        if click.confirm('build and/or dist folder found which will be deleted. Continue?'):
            shutil.rmtree(cwd / 'build')
            shutil.rmtree(cwd / 'dist')
        else:
            click.Abort()
            return

    shell(find_python() + ' -m pip install --upgrade wheel twine', silent=False)
    shell(find_python() + ' setup.py sdist bdist_wheel', silent=False)

    do_twine()


def deploy_github(url, main):
    shell('git remote add origin ' + url, silent=False)
    shell('git branch -M ' + main, silent=False)
    shell('git push -u origin ' + main, silent=False)


def do_twine():
    """Login and upload to PyPi."""
    click.echo('Log in to PyPi...')
    shell(find_python() + ' -m twine upload *', 'dist', silent=False)



import pathlib
import os
import datetime

import click

from mpl_pymanage.add import add_package, add_license, add_github_action
from mpl_pymanage.create import create_project, create_docs
from mpl_pymanage.utils import confirm_copy, find_project_data
import mpl_pymanage.utils as utils
from mpl_pymanage.validators import validate_project_name
from mpl_pymanage.deploy import deploy_pypi, deploy_github
from mpl_pymanage.run import run_tests, run_coverage, run_docs, run_flake8


@click.group('mpl_pymanage')
def cli():
    """\b
██████╗ ██╗   ██╗███╗   ███╗ █████╗ ███╗   ██╗ █████╗  ██████╗ ███████╗
██╔══██╗╚██╗ ██╔╝████╗ ████║██╔══██╗████╗  ██║██╔══██╗██╔════╝ ██╔════╝
██████╔╝ ╚████╔╝ ██╔████╔██║███████║██╔██╗ ██║███████║██║  ███╗█████╗
██╔═══╝   ╚██╔╝  ██║╚██╔╝██║██╔══██║██║╚██╗██║██╔══██║██║   ██║██╔══╝
██║        ██║   ██║ ╚═╝ ██║██║  ██║██║ ╚████║██║  ██║╚██████╔╝███████╗
╚═╝        ╚═╝   ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝  ╚═══╝╚═╝  ╚═╝ ╚═════╝ ╚══════╝

    A command line interface (CLI) for managing Python projects at MPL.

"""
    pass


@cli.command('create')
@click.option('-n', '--name', 'name', required=True, callback=validate_project_name,
              prompt='Project name', help='Name of the project, same as the main package.')
@click.option('-G', '--no-git', 'no_git', is_flag=True, default=False,
              help='Do not create git repository')
@click.option('-I', '--no-install', 'no_install', is_flag=True, default=False,
              help='Do not pip-install in editable mode')
@click.option('-a', '--author', 'author', default=lambda: os.environ.get('USER', 'AUTHOR'), prompt='Author')
@click.option('-d', '--description', 'description', default='', prompt='(Short) description')
@click.option('-l', '--license', 'license', type=click.Choice([
    'MIT', 'GPLv3', 'Apache', 'None'
], case_sensitive=False), prompt='License', default='None')
def create_project_command(name, no_git, no_install, author, description, license):
    """Create a standard Python project structure.

    By default, the project is also pip-installed for development
    in editable mode, and a local git repository is also created.
    """

    create_project(name, no_git, no_install, author, description, license)


@cli.group('deploy')
def deploy(*args, **kwargs):
    """Deploy your project."""
    pass


@deploy.command('pypi')
def deploy_pypi_command():
    """Deploy project to PyPi package repository.

Remember to adjust the version, email and url in setup.py before submitting.
"""
    deploy_pypi()


@deploy.command('repo')
@click.option('-u', '--url', 'url', required=True, prompt='URL of the remote repository (github, gitlab, etc.)?',
              help='URL of the remote repository (github, gitlab, etc.)')
@click.option('-m', '--main-branch', 'main', required=True, default='main',
              prompt='Default branch (main, master, etc.)?',
              help='The default remote branch')
def deploy_repo_command(url, main):
    """Deploy local git repository to github, gitlab, bitbucket, etc."""
    deploy_github(url, main)


@cli.group('run')
def run(*args, **kwargs):
    """Run tests or analyze test coverage"""
    pass


@run.command('tests')
@click.option('-t', 'directory', required=True, prompt='Test folder', help='Directory with tests.',
              default=lambda: find_project_data().get('tests', None))
@click.option('-r', '--runner', 'runner', required=True, default='unittest',
              type=click.Choice(['unittest', 'pytest'], case_sensitive=False),
              help='Test runner', prompt='Test runner')
def run_tests_command(directory, runner):
    """Run tests in a test folder."""
    run_tests(directory, runner)


@run.command('coverage')
@click.option('-s', '--source', 'source', required=True, help='Package on which to run coverage.',
              prompt='Source package',
              default=lambda: find_project_data().get('package', None))
@click.option('-t', '--tests', 'test_dir', required=True,
              prompt='Test folder', help='Directory with tests.',
              default=lambda: find_project_data().get('tests', None))
@click.option('-r', '--runner', 'runner', required=True, default='unittest',
              type=click.Choice(['unittest', 'pytest'], case_sensitive=False),
              help='Test runner', prompt='Test runner')
def run_coverage_command(source, test_dir, runner):
    """Run test coverage."""
    run_coverage(source, test_dir, runner)


@run.command('docs')
def run_docs_command():
    """Build the documentation and show it in browser."""
    run_docs()


@run.command('flake8')
@click.argument('dirs', nargs=-1)
def run_flake8_command(dirs):
    """Run PEP8 style enforcement.

But in contrast to PEP8, by default 120 characters per line are ok.
"""

    # If user forgets to give directories to scan, make a guess.
    if len(dirs) == 0:
        guess = find_project_data().get('package')
        if guess:
            if click.confirm('No directory to scan was given. Did you mean "%s"?' % guess, default=True):
                run_flake8([guess])
                return
        raise click.BadParameter('No directories to scan.')

    run_flake8(dirs)


@cli.group('add')
def add(*args, **kwargs):
    """Adds something to the current project."""
    pass


@add.command('docs')
def add_docs_command():
    """Add a docs folder with Sphinx documentation to the current directory."""
    cwd = pathlib.Path.cwd()
    create_docs(cwd, 'PROJECT_NAME', 'AUTHOR')


@add.command('package')
@click.argument('package_name')
def add_package_command(package_name):
    """Add a package to the current directory.

    PACKAGE_NAME must be the fully qualified package name, e.g.

         mpl_pymanage add package myproject.mypackage
    """
    click.echo('Create package...', nl=False)
    add_package(package_name)
    click.echo('Done.')


@add.command('license')
@click.option('-n', '--name', 'name', type=click.Choice(['MIT', 'GPLv3', 'Apache', 'None'], case_sensitive=False),
              required=True, prompt='License')
def add_license_command(name):
    """Add a license to the current project."""
    add_license(pathlib.Path.cwd(), name)


@add.command('gitignore')
def add_gitignore_command():
    """Add a default .gitignore file to the current directory."""
    cwd = pathlib.Path.cwd()
    target = cwd / '.gitignore'
    source = pathlib.Path(__file__).parent / 'templates' / '.gitignore'

    confirm_copy(source, target)


@add.command('setup.py')
@click.option('-a', '--author', 'author', default=lambda: find_project_data().get('author'), prompt='Author')
@click.option('-e', '--email', 'email', prompt='Author email', default=lambda: find_project_data().get('email'))
@click.option('-n', '--name', 'name', prompt='Package name', required=True,
              default=lambda: find_project_data().get('package'))
@click.option('-v', '--version', 'version', prompt='Version', default='0.0.1', required=True)
@click.option('-d', '--description', 'description', prompt='Short description', default='', required=True)
@click.option('-u', '--url', 'url', prompt='URL', default='', required=True)
def add_setup_py_command(author, email, name, version, description, url):
    """Add a setup.py file to the current directory"""
    cwd = pathlib.Path.cwd()
    target = cwd / 'setup.py'
    source = pathlib.Path(__file__).parent / 'templates' / 'setup.py'

    confirm_copy(source, target)

    if not email:
        email = 'EMAIL'

    utils.substitute(target, {
        'AUTHOR': author,
        'EMAIL': email,
        'PROJECT_NAME': name,
        'VERSION': version,
        'DESCRIPTION': description,
        'URL': url,
        'YEAR': str(datetime.date.today().year)
    })


@add.command('github-action')
@click.option('-p', '--package', 'package', required=True, prompt='Package',
              default=lambda: find_project_data().get('package', None))
@click.option('-t', '--tests', 'tests', required=True, prompt='Test folder',
              default=lambda: find_project_data().get('tests', None))
def add_github_action_command(package, tests):
    """Add github default action"""
    add_github_action(package, tests)


@cli.command('info')
def info_command():
    """Scan for project data."""

    info = find_project_data()
    info['python'] = utils.find_python()
    info['sphinx-quickstart'] = utils.find_sphinx_quickstart()
    info['sphinx-build'] = utils.find_sphinx_build()

    def echo_warning(title, key):
        value = info.get(key, 'NOT FOUND')
        if value is None:
            value = 'NOT FOUND'
        if value == 'NOT FOUND':
            click.echo(title + value, nl=False)
            click.secho(' [!]', fg='red')
        else:
            click.echo(title + value)

    echo_warning('Project name: ', 'name')
    echo_warning('Package: ', 'package')
    echo_warning('Test directory: ', 'tests')
    echo_warning('Version: ', 'version')
    echo_warning('Author: ', 'author')
    echo_warning('Email: ', 'email')
    echo_warning('URL: ', 'url')
    echo_warning('Python: ', 'python')
    echo_warning('sphinx-quickstart: ', 'sphinx-quickstart')
    echo_warning('sphinx-build: ', 'sphinx-build')


if __name__ == '__main__':
    cli()



import functools
import os
import re
import shutil
import subprocess
from pathlib import Path
from os.path import basename, exists

import click


class NotFoundException(Exception):
    pass


def task(msg_before, msg_after=None):
    def outer_wrapper(func):
        @functools.wraps(func)
        def inner_wrapper(*args, **kwargs):
            click.echo(msg_before, nl=not msg_after)
            try:
                result = func(*args, **kwargs)
                if msg_after:
                    click.secho(msg_after, fg='green')
                return result
            except Exception as e:
                click.secho('ERROR', fg='red')
                raise e
        return inner_wrapper
    return outer_wrapper


def substitute(path, replacement_dict):
    with open(path, 'r') as f:
        content = f.read()

    for pattern, subs in replacement_dict.items():
        content = re.sub(pattern, subs, content)

    with open(path, 'w') as f:
        f.write(content)


def replace(path, subs):
    with open(path, 'r') as f:
        content = f.read()

    if content:
        for key, value in subs.items():
            content = content.replace(key, value)
        with open(path, 'w') as f:
            f.write(content)


def shell(cmd, root=None, silent=True, **kwargs):
    """ Silently perform a shell command.

    Parameters
    ----------
    cmd: str
        The command to perform.

    root: path or str
        The directory where to perform the command. Default: current directory.
    """
    if root:
        cmd = 'cd ' + root + '&& ' + cmd
    if silent:
        return subprocess.run(cmd, shell=True, stdout=subprocess.DEVNULL, **kwargs)
    return subprocess.run(cmd, shell=True, **kwargs)


def render(path, template=None, subs=None):
    """ Write a file based on a file or string template.

    Parameters
    ----------
    path: str or Pathlib path
        The path of the file to write.

    template: Path or str
        The file path or the string used as template.

    subs: dict
        The substitutions to perform on the template.
    """
    if template:
        if os.path.exists(template):
            with open(template, 'r') as f:
                content = f.read()
        else:
            content = template
    else:
        content = ''

    if subs:
        for key, value in subs.items():
            content = content.replace(key, str(value))

    with open(path, 'w') as f:
        f.write(content)


def confirm_copy(source, target):
    if not os.path.exists(target):
        shutil.copyfile(source, target)
    else:
        if click.confirm('%s file already exists in current directory. Overwrite?' % basename(target)):
            shutil.copyfile(source, target)


def file_content(path):
    if not os.path.exists(path):
        return ''

    with open(path, 'r') as f:
        content = f.read()
    return content


def find_project_data():
    """Try to find relevant project data.

    Should be executed from the project root directory (the one with the setup.py).
    """
    cwd = Path.cwd()

    # parse setup.py

    def find_parameter(par_name, text):

        pattern = re.compile(par_name + r'\s*=\s*([^,)}\n]+)')
        matches = pattern.findall(text)
        parameter = None
        for m in matches:
            result = re.search("'([^']*)'", m)
            if result:
                parameter = result.group(1)
            result = re.search('"([^"]*)"', m)
            if result:
                parameter = result.group(1)
            if parameter is not None:
                break
        return parameter

    setup_py = file_content(cwd / 'setup.py')

    info = dict()
    for key in ['name', 'url', 'version', 'email', 'author']:
        value = find_parameter(key, setup_py)
        if value is not None:
            info[key] = value

    if info.get('name') and exists(cwd / info['name']):
        info['package'] = info['name']

    test_dir = cwd / 'tests'
    if exists(test_dir) and os.path.isdir(test_dir):
        info['tests'] = 'tests'

    if 'tests' not in info:
        test_dir = cwd / 'test'
        if exists(test_dir) and os.path.isdir(test_dir):
            info['tests'] = 'test'

    # Try to find author if not yet found and email
    if 'author' not in info:
        result = shell('git config user.name', capture_output=True, silent=False)

        if result.returncode == 0:
            info['author'] = result.stdout.decode('utf-8').strip()

    if 'author' not in info:
        author = os.environ.get('USER')
        if author:
            info['author'] = author

    if 'author' not in info:
        author = os.environ.get('USERNAME')
        if author:
            info['author'] = author

    if 'email' not in info:
        result = shell('git config user.email', capture_output=True, silent=False)
        if result.returncode == 0:
            info['email'] = result.stdout.decode('utf-8').strip()

    return info


def find_python():
    found = shutil.which('python')
    if not found:
        found = shutil.which('py')
    return found


def find_sphinx_quickstart():
    return shutil.which('sphinx-quickstart')


def find_sphinx_build():
    return shutil.which('sphinx-build')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from os.path import exists
from setuptools import setup, find_packages

author = 'Matthias Baer'
email = 'matthias.baer@mpl.mpg.de'
description = 'A CLI for managing Python projects.'
name = 'mpl_pymanage'
year = '2021'
version = '0.1.0'

setup(
    name=name,
    author=author,
    author_email=email,
    url='https://github.com/MPL-IT/mpl_pymanage',
    version=version,
    packages=find_packages(),
    package_dir={name: name},
    include_package_data=True,
    entry_points='''
        [console_scripts]
        mpl_pymanage=mpl_pymanage.mpl_pymanage:cli
    ''',
    license="GPL v3",
    description=description,
    long_description=open('README.md').read() if exists('README.md') else '',
    long_description_content_type="text/markdown",
    install_requires=['Click',
                      'sphinx'
                      ],
    # not to be confused with definitions in pyproject.toml [build-system]
    python_requires=">=3.6",
    keywords=['CLI', 'cli', 'Python project management'],
    classifiers=['Operating System :: OS Independent',
                 'Programming Language :: Python :: 3',
                 'Intended Audience :: Developers',
                 'Environment :: Console'
                 ],
    platforms=['ALL'],
)
